@@ -1,4 +1,5 @@
 // vim:ts=4:sw=4:expandtab
+#include <config.h>
 #include <string.h>
 #include <limits.h>
 #include <stdio.h>
@@ -14,7 +15,7 @@
 
 #include "i3status.h"
 
-#if defined(LINUX)
+#if defined(__linux__)
 #include <linux/ethtool.h>
 #include <linux/sockios.h>
 #define PART_ETHSPEED "E: %s (%d Mbit/s)"
@@ -32,8 +33,7 @@
 #endif
 
 static int print_eth_speed(char *outwalk, const char *interface) {
-#if defined(LINUX)
-    /* This code path requires root privileges */
+#if defined(__linux__)
     int ethspeed = 0;
     struct ifreq ifr;
     struct ethtool_cmd ecmd;
@@ -43,7 +43,7 @@ static int print_eth_speed(char *outwalk, const char *interface) {
     ifr.ifr_data = (caddr_t)&ecmd;
     (void)strcpy(ifr.ifr_name, interface);
     if (ioctl(general_socket, SIOCETHTOOL, &ifr) == 0) {
-        ethspeed = (ecmd.speed == USHRT_MAX ? 0 : ecmd.speed);
+        ethspeed = (ecmd.speed == USHRT_MAX ? 0 : ethtool_cmd_speed(&ecmd));
         return sprintf(outwalk, "%d Mbit/s", ethspeed);
     } else
         return sprintf(outwalk, "?");
@@ -135,6 +135,8 @@ static int print_eth_speed(char *outwalk, const char *interface) {
  * | 127.0.0.1    | ::1/128      | IPv4      | ok                |
  */
 void print_eth_info(yajl_gen json_gen, char *buffer, const char *interface, const char *format_up, const char *format_down) {
+    const char *format = format_down;  // default format
+
     const char *walk;
     char *outwalk = buffer;
 
@@ -158,7 +160,6 @@ void print_eth_info(yajl_gen json_gen, char *buffer, const char *interface, cons
     if (ipv4_address == NULL) {
         if (ipv6_address == NULL) {
             START_COLOR("color_bad");
-            outwalk += sprintf(outwalk, "%s", format_down);
             goto out;
         } else {
             prefer_ipv4 = false;
@@ -167,28 +168,36 @@ void print_eth_info(yajl_gen json_gen, char *buffer, const char *interface, cons
         prefer_ipv4 = false;
     }
 
+    format = format_up;
+
     const char *ip_address = (prefer_ipv4) ? ipv4_address : ipv6_address;
     if (BEGINS_WITH(ip_address, "no IP")) {
         START_COLOR("color_degraded");
     } else {
         START_COLOR("color_good");
     }
-    for (walk = format_up; *walk != '\0'; walk++) {
+
+out:
+    for (walk = format; *walk != '\0'; walk++) {
         if (*walk != '%') {
             *(outwalk++) = *walk;
-            continue;
-        }
 
-        if (BEGINS_WITH(walk + 1, "ip")) {
+        } else if (BEGINS_WITH(walk + 1, "ip")) {
             outwalk += sprintf(outwalk, "%s", ip_address);
             walk += strlen("ip");
+
         } else if (BEGINS_WITH(walk + 1, "speed")) {
             outwalk += print_eth_speed(outwalk, interface);
             walk += strlen("speed");
+
+        } else if (BEGINS_WITH(walk + 1, "interface")) {
+            outwalk += sprintf(outwalk, "%s", interface);
+            walk += strlen("interface");
+
+        } else {
+            *(outwalk++) = '%';
         }
     }
-
-out:
     END_COLOR;
     free(ipv4_address);
     free(ipv6_address);
